% Creates the simulated "microbial communities" composed of species that consist of a set of "genes"
% used to model metagenomic sequencing in the absence of sequencing and annotation error.
function createCollection()

  global CommunityOverwrite;
  if isempty(CommunityOverwrite)
    CommunityOverwrite = 0;
  end
  
  % Inputs

  nGenes=100;
  nSamples=100;
  nReads=5000000;

  Collection=sprintf('C.%d_Genes.%d_Samples',nGenes,nSamples);
  
  % Fixed
  geneMinLength = 400;
  geneMaxLength = 5000;
  doublingProb = 0.05;
  GeneMinAbundance = 0.2;
  GeneMaxAbundance = 0.8;

  % For making the genes
  ConstantGeneLength = 1500;

  RelativeAbundance = 4;

  vStart = 0.05;
  vEnd = 0.2;
  nV = 5;
  variance = [0 logspace(log10(vStart),log10(vEnd),nV) 1];
  nV = nV+2;
  oStart = 20;
  oEnd = 100;
  nO = 5;
  ListnOTUs = linspace(oStart,oEnd,nO);
  OTUStep = (oEnd-oStart) / (nO-1);
  
  % Make the directory structure
  if (exist(Collection, 'dir'))
    sprintf('Collection %s already exists!', Collection)
    if ~ CommunityOverwrite
      return
    end
  else
    mkdir(Collection);
  end

  % --- Write out the parameter data --- %
  fileName = sprintf('%s/Parameters.mat',Collection);
  save(fileName, 'nGenes', 'nSamples', 'geneMinLength', 'geneMaxLength', 'GeneMinAbundance', 'GeneMaxAbundance', 'RelativeAbundance', 'nReads','oStart','oEnd','nO','vStart','vEnd','nV','doublingProb');

  % --- Create and Sample the Ecosystems --- %
  collectionFile = sprintf('%s/Communities.txt',Collection);
  FH = fopen(collectionFile, 'w');
  fclose(FH);

  % --- --- --- Genes --- --- --- %

  % Lengths
  genes=randi([geneMinLength, geneMaxLength], 1, nGenes-1);
  genes(nGenes) = ConstantGeneLength;

  % Abundances
  geneMeanAbundances = GeneMinAbundance + (GeneMaxAbundance-GeneMinAbundance)*rand(1,nGenes-1);
  geneMeanAbundances(nGenes) = 1;

  % So that I don't end up with OTUs with no genes
  MinOTUSize = nGenes * GeneMinAbundance * 0.5;

  AllOTUs = [];
  % --- Make OTUs --- %
  
  % OTUStep new OTUs are made for each iteration
  
  for io=1:nO
    nOTUs = OTUStep;
    geneAbundances = round (nOTUs * geneMeanAbundances);
    
    % --- Make OTUs --- %
    
    OTUs = zeros(nOTUs,nGenes);
    otuSize = zeros(1,nOTUs);
    for j=1:nGenes
      
      geneOTUs = randperm(nOTUs);
      
      for i=1:geneAbundances(j)
	otu = geneOTUs(i);
	otuSize(otu) = otuSize(otu) + 1;
	
	% Multiple copies?
	if j ~= nGenes % don't do this for the "16S"
	  OTUs(otu,j) = genes(j);
	  prob = doublingProb;
	  while rand < prob
	    OTUs(otu,j) = OTUs(otu,j) + genes(j);
	    prob = prob*doublingProb; % gets exponentially smaller
	  end
	else
	  OTUs(otu,j) = genes(j);
	end
      end
      
    end
    
    % Make sure that all my OTUs are at least a minimum size
    % stealing from the largest if this is not the case
    tooSmall = find(otuSize < MinOTUSize);
    for i=1:length(tooSmall)
      otu=tooSmall(i);
      while otuSize(otu) < MinOTUSize
	[~, loser] = max(otuSize);
	
	mix = find( OTUs(loser,:) > 0 );
	mix = randperm(length(mix));
	
	fprintf('Too few %d %d\n', otu, otuSize(otu));
	
	for j=1:length(mix)
	  
	  % Does this exist in our otu?
	  if OTUs(otu,mix(j)) == 0
	    
	    % Swap the value -- important for dups
	    OTUs(otu,mix(j)) = OTUs(loser,mix(j));
	    OTUs(loser,mix(j)) = 0;
	    
	    % Change the counts
	    otuSize(otu) = otuSize(otu) + 1;
	    otuSize(loser) = otuSize(loser) - 1;
	    
	    break;
	    
	  end
	end
	
      end
    end
    
    AllOTUs = [AllOTUs; OTUs];
        
    % --- / make OTUs --- %
    
  end
  
  
  % --- Assemble Communities --- %
  for io=1:nO
    nOTUs = ListnOTUs(io);
    
    OTUs = AllOTUs(1:nOTUs,:);
    
    % Generate the Mean abundances for each OTU
    MeanAbundances = 10.^( RelativeAbundance * rand(1,nOTUs) );
    
    
    for iv=1:nV
      varAbundance = variance(iv);

      % Generate the variations in OTU abundance
      cOTUs = zeros(nSamples,nOTUs);
      for i=1:nSamples
	if iv<nV
	  for j=1:nOTUs
	    cOTUs(i,j) = MeanAbundances(j) * 10^( variance(iv) * 3 * randn );
	  end
	else
	  % Generate the Mean abundances for each nOTU
	  cOTUs(i,:) = 10.^( RelativeAbundance * rand(1,nOTUs));
	end
      end
      
      % Calculate the genes in each community
      cGenes = zeros(nSamples,nGenes);
      
      for i=1:nSamples;
	for j=1:nOTUs
	  for k=1:nGenes
	    cGenes(i,k) = cGenes(i,k) + OTUs(j,k) * cOTUs(i,j); %gene length * abundance
	  end
	end
      end
      
      % --- Generate a Single Community --- %
      
      % Generate the community and name
      name = sprintf('C.%d_OTU.%.2f_var.1', nOTUs, variance(iv));
      Community = sprintf('%s/%s', Collection, name);
      fprintf('%s\n', Community);
      
      % Keep a list of the Communities we create
      FH = fopen(collectionFile, 'a');
      fprintf(FH,'%s\n', name);
      fclose(FH);
      
      mkdir(Community);
      
      % Write out the parameters
      fileName = sprintf('%s/Parameters.mat',Community);
      save(fileName, 'nGenes', 'nOTUs', 'nSamples', 'geneMinLength', 'geneMaxLength', 'doublingProb', 'RelativeAbundance', 'varAbundance', 'MinOTUSize', 'ConstantGeneLength');
      
      % Write out the genes
      fileName = sprintf('%s/Genes.mat',Community);
      FH = fopen(fileName, 'w');
      fwrite(FH,genes,'double'); %randi returns doubles
      fclose(FH);
      
      % Write out the gene abundances
      fileName = sprintf('%s/GeneAbundances.mat',Community);
      FH = fopen(fileName, 'w');
      fwrite(FH,geneAbundances,'double');
      fclose(FH);

      % Write out the gene mean abundances
      geneAbundances = round (nOTUs * geneMeanAbundances); % for memories sake
      fileName = sprintf('%s/GeneMeanAbundances.mat',Community);
      FH = fopen(fileName, 'w');
      fwrite(FH,geneMeanAbundances,'double');
      fclose(FH);

      % Write out the Mean OTU Abundances
      fileName = sprintf('%s/MeanAbundances.mat', Community);
      FH = fopen(fileName, 'w');
      fwrite(FH,MeanAbundances,'double');
      fclose(FH);    
      
      % Write out the Actual Abundances
      fileName = sprintf('%s/SampleOTUs.mat',Community);
      FH = fopen(fileName, 'w');
      fwrite(FH,cOTUs,'double');
      fclose(FH);

      % Write out the OTUs
      fileName = sprintf('%s/OTUs.mat',Community);
      FH = fopen(fileName, 'w');
      fwrite(FH,OTUs,'double');
      fclose(FH);
      
      % Write out the genes in each community
      fileName = sprintf('%s/SampleGenes.mat',Community);
      FH = fopen(fileName, 'w');
      fwrite(FH,cGenes,'double');
      fclose(FH);

      
    end
  end


end
