#!/usr/bin/perl -w
# Auth: Rogan
#  This version allows multiple KOs per gene.
use strict;
use Getopt::Long;

my ($path,$KEGGVersion);
my $koMethod;
my $kKey;
my $kValue;
my $kWeight;
my $geneFile;
my $ignoreUnknownGenes;

GetOptions("komethod=s"=>\$koMethod,"key=i"=>\$kKey,"value=i"=>\$kValue,"weightedKOs"=>\$kWeight,"genelengths=s"=>\$geneFile,"ignoreunknowngenes"=>\$ignoreUnknownGenes);

use KEGG qw( loadKOs KEGGSort getAverageKOLengths getNormalizedGeneLengths );
use Lookup qw( createLookupTable createLookupTableWeighted );

my $dataFile = shift;
my $outPrefix = shift;

unless ( defined $dataFile ) {
  print $0, " inFile\n";
  exit;
}

unless (defined $koMethod) {
  $koMethod='';
}

# Default behavior
unless ( defined $outPrefix ) {
  $outPrefix = "${dataFile}.highestGenes";
}

# If the file is a gzip, we unzip; otherwise it's untouched
if ($dataFile =~ /gz$/) {
  $dataFile = "zcat $dataFile |";
  $outPrefix =~ s/(\.gz)//;
}

unless (defined $ignoreUnknownGenes) {
  $ignoreUnknownGenes=0;
}

# --- Load KOs --- #

my %geneToKO;
if ( $koMethod eq '') {
  %geneToKO = %{ loadKOs() };
} else {
  
  # The method is the filename
  unless ( -e $koMethod ) {
    print "Cannot locate ko-file $koMethod\n";
    exit;
  }
  # Make sure we have keys and values
  unless (defined $kKey) {
    $kKey=0;
  } 
  unless (defined $kValue) {
    $kValue=1;
  }
  
  print "Getting KOs from a lookup table: $koMethod\n";
 
  unless (defined $kWeight) {
    # We expect a list of KOs for each gene
    %geneToKO = %{ createLookupTable($koMethod, $kKey, $kValue, 1) };
  } else {
    %geneToKO = %{ createLookupTableWeighted($koMethod) };
    print "Using a weighting scheme.\n";
  }

}

my %geneToLength;
unless (defined $geneFile) {
  %geneToLength = %{ getNormalizedGeneLengths() };
} else {
  %geneToLength = %{ createLookupTable($geneFile,0,1) };
}

# --- ---  --- --- #


# --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- #
#   Read in BLAST results; record best-hit KOs and KO counts
# --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- #

my $outFile = ${outPrefix} . '.matches';
open (MATCH, ">$outFile") or die "Cannot write to $outFile\n";
print MATCH "# KO\tHitRank\tQueryID\tTargetID\tIdentity\tEval\tBitScore\tScore\n";

my %KOcounts;
my %KOcopies;
my %KOCoverage;
# Initiate the loop variables
my $eVal = 1;
my $seqName = '';
my @geneNames = '';
my @KOs;
my $hits;
# Keep track of the stats
my $unmatched=0;
my $matched=0;
my $reads=0;
my $missed=0;
my $nblasts=0;
my $foundKO=0;

# Note: No <; we may need to pipe our input
open (FH, $dataFile) or die "Cannot read $dataFile\n";
while (<FH>) {	
  my $line=$_; 
  chomp($line);

  # Skip comments
  if ( $line =~ /^#/ ) {
    if ( $line =~ /^# 0 hits found$/ ) {
      $missed++;
    } elsif ( $line =~ /^# BLAST processed \d+ queries/ ) {
      my ($sum) = $line =~ /^# BLAST processed (\d+) queries/;
      $nblasts+=$sum;
    }
    next;
  }
  
  my @fields = split(/\t/, $line);
  ## BLAST FIELDS: [0]qseqid [1]sallseqid [2]pident [3]evalue [4]bitscore [5]score

  # Watch out for garbage lines
  unless ( defined $fields[0] ) {
    print "Garbage: $line\n";
    next;
  }

  # 1. The top hit is the best.
  # 2. The top hit is a tie.
  # 3. The top hit has no KO.


  #        Old Sequence,              Diff. E-Value,       Already Annotated
  next if ($fields[0] eq $seqName && $fields[3] != $eVal && $hits > 0 );

  # Bounce unknown 
  next if ( $ignoreUnknownGenes && ! defined $geneToLength{$fields[1]} );
  
  #   New Sequence                  
  if ( $fields[0] ne $seqName ) {
    $reads++;
    # Was the previous just not matched?
    if ( ! $foundKO && $seqName ne '' ) {
      print MATCH "# $seqName ", "\n";#, join(' ', @geneNames), "\n"; # hide it from further processing, though
      $unmatched++;
    } elsif ( scalar @KOs > 0 ) {
      $matched++;
    }
    
    # Write out the KOs and unset the array
    for (my $i=0; $i<scalar @KOs; $i++) { 
      
      if ( ref($KOs[$i]) eq 'ARRAY' ) {
	my $portion = 1 / (scalar @{$KOs[$i]});
	my %found;
	foreach my $ko (@{$KOs[$i]}) {
	  $KOcounts{$ko} += $portion/ (scalar @KOs);
	  # On the copies level, they are all a full copy, so make sure I only count once
	  unless ( defined $found{$ko} ) {
	    $KOcopies{$ko} += 1/(scalar @KOs)/$geneToLength{$geneNames[$i]};
	    $KOCoverage{$ko} += 1/(scalar @KOs);
	    $found{$ko}++;
	  }
	}
      } elsif ( ref($KOs[$i]) eq 'HASH' ) {
	# The portions are already known
	foreach my $ko (keys %{$KOs[$i]}) {
	  $KOcounts{$ko} +=  $KOs[$i]->{$ko} / (scalar @KOs);
	  # On the copies level, they are all a full copy.
	  $KOcopies{$ko} += 1/(scalar @KOs)/$geneToLength{$geneNames[$i]};
	  $KOCoverage{$ko} += 1/(scalar @KOs);
	}
      } else {
	next if ($KOs[$i] eq '');
	$KOcounts{$KOs[$i]} += 1/(scalar @KOs);
	$KOcopies{$KOs[$i]} += 1/(scalar @KOs)/$geneToLength{$geneNames[$i]};
	$KOCoverage{$KOs[$i]} += 1/(scalar @KOs);
      }
    }
    @KOs = ();
    @geneNames = ();
    $hits=0;
    $foundKO=0;
  }
  
  $seqName = $fields[0];
  $eVal = $fields[3];

  # Is there an annotation?  
  push @geneNames, $fields[1];

  my $koName='';
  if (defined $geneToKO{$fields[1]}) {
    $foundKO++;

    push @KOs, $geneToKO{$fields[1]};
    $koName=$geneToKO{$fields[1]};
  
    if ( ref($koName) eq 'ARRAY' ) {
      print MATCH $seqName, "\t", join('|', @{$koName}), "\t", $fields[1], "\t", $fields[3],"\n";
    } elsif ( ref($koName) eq 'HASH' ) {
      my @tmplist;
      foreach my $thisKO (keys %{$koName}) {
	push @tmplist, $thisKO . '=' . $koName->{$thisKO};
      }
      print MATCH $seqName, "\t", join('|', @tmplist), "\t", $fields[1], "\t", $fields[3],"\n";
    } 
    else {
      print MATCH $seqName, "\t", $koName, "\t", $fields[1], "\t", $fields[3],"\n";
    }
  } else {
    push @KOs, '';
    print MATCH $seqName, "\t", $koName, "\t", $fields[1], "\t", $fields[3],"\n";
  }
  
  $hits++;
  
}
# Don't forget the last read!
if ( ! $foundKO && $seqName ne '' ) {
  print MATCH "# $seqName ", "\n";#, join(' ', @geneNames), "\n"; # hide it from further processing, though
  $unmatched++;
} elsif ( scalar @KOs > 0 ) {
  $matched++;
}
for (my $i=0; $i<scalar @KOs; $i++) { 
  if ( ref($KOs[$i]) eq 'ARRAY' ) {
    my $portion = 1 / scalar @{$KOs[$i]};
    my %found;
    foreach my $ko (@{$KOs[$i]}) {
      $KOcounts{$ko} += $portion/(scalar @KOs);
      # Full copy for each KO
      unless ( defined $found{$ko} ) {
	$KOcopies{$ko} += 1/(scalar @KOs)/$geneToLength{$geneNames[$i]};
	$KOCoverage{$ko} += 1/(scalar @KOs);
	$found{$ko}++;
      }
    }
  } elsif ( ref($KOs[$i]) eq 'HASH' ) {
    # The portions are already known
    foreach my $ko (keys %{$KOs[$i]}) {
      $KOcounts{$ko} +=  $KOs[$i]->{$ko} / (scalar @KOs);
      # On the copies level, they are all a full copy.
      $KOcopies{$ko} += 1/(scalar @KOs)/$geneToLength{$geneNames[$i]};
      $KOCoverage{$ko} += 1/(scalar @KOs);
    }
  } else {
    next if ($KOs[$i] eq '');
    $KOcounts{$KOs[$i]} += 1/(scalar @KOs);
    $KOcopies{$KOs[$i]} += 1/(scalar @KOs)/$geneToLength{$geneNames[$i]};
    $KOCoverage{$KOs[$i]}+=1/(scalar @KOs);
  }
}
@KOs = ();

close(FH);

print MATCH "# Found $reads blasted reads, matched $matched (", $matched/$reads,"), missed $unmatched (", $unmatched/$reads, "). $missed reads were not annotated by BLAST out of $nblasts reads\n";
close (MATCH);


# --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- #
#                   Write out the KO Counts
# --- --- --- --- --- --- --- --- --- --- --- --- --- --- --- #

$outFile = ${outPrefix} . '.counts';
open (COUNT, ">$outFile") or die "Cannot write to $outFile\n";

foreach my $ko (sort KEGGSort keys %KOcounts) {
  my ($id) = $ko =~ /^K(\d+)/;
  print COUNT $id, "\t", $KOcounts{$ko}, "\n";
}

close (COUNT);

$outFile = ${outPrefix} . '.copies';
open (COPY, ">$outFile") or die "Cannot write to $outFile\n";

foreach my $ko (sort KEGGSort keys %KOcopies) {
  my ($id) = $ko =~ /^K(\d+)/;
  print COPY $id, "\t", $KOcopies{$ko}, "\n";
}

close (COPY);

%geneToLength=();
my %koToLength = %{ getAverageKOLengths() };
$outFile = ${outPrefix} . '.copiesByKO';
open (COPY, ">$outFile") or die "Cannot write to $outFile\n";
foreach my $ko (sort KEGGSort keys %KOCoverage) {
  my ($id) = $ko =~ /^K(\d+)/;
  print COPY $id, "\t", $KOCoverage{$ko}/$koToLength{$ko}, "\n";
}
close (COPY);

$outFile = ${outPrefix} . '.coverage';
open (COPY, ">$outFile") or die "Cannot write to $outFile\n";
foreach my $ko (sort KEGGSort keys %KOCoverage) {
  my ($id) = $ko =~ /^K(\d+)/;
  print COPY $id, "\t", $KOCoverage{$ko}, "\n";
}
close (COPY);

